package com.energy.manage.service.controller.analysis;

import com.energy.manage.common.base.Page;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.service.config.annotations.UserLoginToken;
import com.energy.manage.service.constant.analysis.AnalysisConstants;
import com.energy.manage.service.controller.base.BaseServiceController;
import com.energy.manage.service.domain.dto.analysis.AnalysisDto;
import com.energy.manage.service.domain.dto.analysis.AnalysisResultDto;
import com.energy.manage.service.domain.vo.analysis.AnalysisResultVo;
import com.energy.manage.service.service.analysis.AnalysisService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 分析
 * @author chy
 * @date 2024/5/23 12:57
 * @desc
 */
@Api(value = "AnalysisController",tags = "分析")
@Slf4j
@RestController
@RequestMapping("/analysis")
public class AnalysisController extends BaseServiceController {

    @Autowired
    private AnalysisService analysisService;


    @UserLoginToken
    @ApiOperation(value = "分析结果列表")
    @GetMapping("/analysisResultList")
    public ResultResp<Page<AnalysisResultVo>> analysisResultList(AnalysisResultDto analysisResultDto)
    {
        analysisResultDto.setRoleId(getRoleId());
        return pageResult(analysisService.analysisResultList(analysisResultDto),analysisResultDto);
    }

    @UserLoginToken
    @ApiOperation(value = "手动分析触发")
    @PostMapping("/analysis")
    public ResultResp analysis(@RequestBody AnalysisDto analysisto)
    {
        //设置为手动分析
        analysisto.setAutoOrManual(AnalysisConstants.MANUAL_ANALYSIS);
        return analysisService.analysis(analysisto);
    }

    @UserLoginToken
    @ApiOperation(value = "分析编辑查询")
    @GetMapping("/analysisEditQuery")
    public ResultResp analysisEditQuery(@RequestParam("batchCode") String batchCode)
    {
        return success(analysisService.analysisEditQuery(batchCode));
    }

    @UserLoginToken
    @ApiOperation(value = "分析详情")
    @PostMapping("/analysisDetail")
    public ResultResp analysisDetail(@RequestParam("batchCode") String batchCode,
                                     @RequestParam("analysisTypeCode") String analysisTypeCode,
                                     @RequestParam("fieldEngineCode") String fieldEngineCode)
    {
        return success(analysisService.analysisDetail(batchCode,analysisTypeCode,fieldEngineCode));
    }

}
