package com.energy.manage.service.controller.datatransfer;

import com.energy.manage.common.base.Page;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.service.config.annotations.UserLoginToken;
import com.energy.manage.service.controller.base.BaseServiceController;
import com.energy.manage.service.domain.dto.datatransfer.DataTransferDto;
import com.energy.manage.service.domain.dto.datatransfer.EditDataTransferTypeDto;
import com.energy.manage.service.domain.vo.datatransfer.DataTransferVo;
import com.energy.manage.service.service.datatransfer.DataTransferService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 数据转换模块儿
 * @author chy
 * @date 2024/5/20 10:17
 * @desc
 */
@Slf4j
@Api(value = "DataTransferController",tags = "数据转换")
@RestController
@RequestMapping("/dataTransfer")
public class DataTransferController extends BaseServiceController {

    @Autowired
    private DataTransferService dataTransferService;


    @UserLoginToken
    @ApiOperation(value = "查询可以数据转换批次")
    @GetMapping("/fieldBatchListForDataTransfer")
    public ResultResp fieldBatchListForDataTransfer(){
        return success(dataTransferService.fieldBatchListForDataTransfer(getRoleId()));
    }

    @UserLoginToken
    @ApiOperation("数据转换列表")
    @GetMapping("/queryDataTransferList")
    public ResultResp<Page<DataTransferVo>> queryDataTransferList(DataTransferDto dataTransferDto){
        dataTransferDto.setRoleId(getRoleId());
        return pageResult(dataTransferService.queryDataTransferList(dataTransferDto),dataTransferDto);
    }


    @UserLoginToken
    @ApiOperation("新增数据转换数据")
    @PostMapping("/addDataTransferList")
    public ResultResp addDataTransferList(@RequestBody DataTransferDto dataTransferDto){
        dataTransferDto.setRoleId(getRoleId());
        dataTransferDto.setUserId(getUserId());
        return dataTransferService.addDataTransfer(dataTransferDto);
    }

    @UserLoginToken
    @ApiOperation("转换/重新转换")
    @PostMapping("/dataTransfer")
    public ResultResp dataTransfer(@RequestBody EditDataTransferTypeDto editDto){
        return dataTransferService.dataTransfer(editDto, getUserId());
    }

    @UserLoginToken
    @ApiOperation("时间粒度编辑")
    @PostMapping("/editTimeGranularity")
    public ResultResp editTimeGranularity(@RequestBody EditDataTransferTypeDto editDto){
        return dataTransferService.editTimeGranularity(editDto.getBatchCode(), editDto.getDataTransferType(), editDto.getTimeGranularity(),getUserId());
    }
}
