package com.energy.manage.service.controller.powerwordcontract;


import com.energy.manage.common.base.Page;
import com.energy.manage.common.enums.IdPrefixEnum;
import com.energy.manage.common.reponse.ResultResp;
import com.energy.manage.service.config.annotations.UserLoginToken;
import com.energy.manage.service.controller.base.BaseServiceController;
import com.energy.manage.service.domain.dto.powerwordcontract.PowerWordContractDelDto;
import com.energy.manage.service.domain.dto.powerwordcontract.PowerWordContractDto;
import com.energy.manage.service.domain.dto.powerwordcontract.PowerWordContractExcelDto;
import com.energy.manage.service.domain.dto.powerwordcontract.PowerWordContractPageDto;
import com.energy.manage.service.domain.vo.powerwordcontract.PowerWordContractPageVo;
import com.energy.manage.service.domain.vo.windfield.WindFieldPageVo;
import com.energy.manage.service.service.powerwordcontract.PowerWordContractService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

@Slf4j
@RestController
@CrossOrigin
@Api(value = "PowerWordContractController", tags = "合同功率曲线接口")
@RequestMapping(value = "/api/powerword/contract")
public class PowerWordContractController extends BaseServiceController {

    @Autowired
    private PowerWordContractService powerWordContractService;


    @UserLoginToken
    @PostMapping(value = "/powerWordContractImportData")
    @ApiOperation(value = " 同步合同功率,（选择风场编号，导入功率曲线）")
    public ResultResp powerWordContractImportData(@RequestPart("file") MultipartFile file,
                                                  @RequestParam("fieldCode") String fieldCode) {
        if(!fieldCode.contains(IdPrefixEnum.WIND_FIELD_NUMBER.getCode())){
            return ResultResp.FAIL("风场编号不正确，请确认数据！");
        }
        PowerWordContractExcelDto powerWordContractExcelDto = new PowerWordContractExcelDto();
        powerWordContractExcelDto.setFieldCode(fieldCode);
        powerWordContractExcelDto.setCreateBy(getUserId());
        return powerWordContractService.powerWordContractImportData(file, powerWordContractExcelDto);
    }


    @UserLoginToken
    @PostMapping(value = "/powerWordContractPage")
    @ApiOperation(value = " 同步合同功率曲线分页查询")
    public ResultResp<Page<PowerWordContractPageVo>> powerWordContractPage(@RequestBody PowerWordContractPageDto powerWordContractPageDto) {

        Page<PowerWordContractPageVo> page = powerWordContractService.powerWordContractPage(powerWordContractPageDto);
        return ResultResp.SUCCESS(page);
    }


    @UserLoginToken
    @PostMapping(value = "/delPowerWordContract")
    @ApiOperation(value = "删除风机合同功率曲线")
    public ResultResp delPowerWordContract(@RequestBody PowerWordContractDelDto powerWordContractDelDto) {

        boolean flg = powerWordContractService.delPowerWordContract(powerWordContractDelDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }


    @UserLoginToken
    @PostMapping(value = "/updatePowerWordContract")
    @ApiOperation(value = "编辑合同功率曲线")
    public ResultResp updatePowerWordContract(@RequestBody PowerWordContractDto powerWordContractDto) {
        boolean flg = powerWordContractService.updatePowerWordContract(powerWordContractDto);
        return flg ? ResultResp.SUCCESS() : ResultResp.FAIL();
    }


}
